/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

const { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");

function _log(aMessage) {
	Services.console.logStringMessage("[QueryInterface Module]: "+aMessage);
}

function TemplateResultSet(aStatement) {
	this._statement = aStatement;
	this._columnNames = new Array();
	let count = this._statement.columnCount;
	for (let i=0; i<count; ++i) {
		this._columnNames.push("?"+this._statement.getColumnName(i));
	}
}

TemplateResultSet.prototype = {
	hasMoreElements: function() {
		if (!this._statement)
			return false;
		
		let result = this._statement.executeStep();
		if (!result)
			this._statement = null;
		
		return result;
	},
	
	getNext: function() {
		return new TemplateResult(this);
	},
	
	getColumnIndex: function(aColumnName) {
		for (let i=0; i<this._columnNames.length; ++i) {
			if (this._columnNames[i] == aColumnName) {
				return i;
			}
		}
		return -1;
	},
	
	fillColumnValues: function(aArray) {
		if (!this._statement)
			return;
		
		for (let i=0; i<this._columnNames.length; ++i) {
			let value;
			let type = this._statement.getTypeOfIndex(i);
			if (type == this._statement.VALUE_TYPE_INTEGER) {
				value = this._statement.getInt64(i);
			} else if (type == this._statement.VALUE_TYPE_FLOAT) {
				value = this._statement.getDouble(i);
			} else {
				value = this._statement.getString(i);
			}
			aArray.push(value);
		}
	},
	
	QueryInterface: XPCOMUtils.generateQI([Ci.nsISimpleEnumerator])
};

function TemplateResult(aResultSet) {
	let rdfService = Cc['@mozilla.org/rdf/rdf-service;1'].getService(Ci.nsIRDFService);
	this._node = rdfService.GetAnonymousResource();
	this._resultSet = aResultSet;
	this._values = new Array();
	if (aResultSet)
		this._resultSet.fillColumnValues(this._values);
}

TemplateResult.prototype = {
	isContainer: false,
	isEmpty: true,
	mayProcessChildren: false,
	type: "",
	get id() {
		return this._node.Value;
	},
	get resource() {
		return this._node;
	},
	// return the value of that bound variable such as ?name
	getBindingFor: function(aVar) {
		//_log("getBindingFor");
		if (!aVar)
			throw Cr.NS_ERROR_INVALID_POINTER;
			
		if (!this._resultSet)
			return "";
		
		let index = this._resultSet.getColumnIndex(aVar);
		if (index < 0)
			return "";
		
		return this._values[index];
	},
	
	// return an object instead of a string for convenient comparison purposes
	// or null to say just use string value
	getBindingObjectFor: function(aVar) {
		//_log("getBindingObjectFor");
		return null;
	},
	
	// called when a rule matches this item.
	ruleMatched: function(aQuery, aRuleNode) { },
	
	// the output for a result has been removed and the result is no longer being used by the builder
	hasBeenRemoved: function() { },
	
	QueryInterface: XPCOMUtils.generateQI([Ci.nsIXULTemplateResult])
};

function QueryProcessor() {
	this._generationStarted = false;
}

QueryProcessor.prototype = {
	
	/***** ***** nsIXULTemplateQueryProcessor ***** *****/
	getDatasource: function QP_getDatasource(aDataSources, aRootNode, aIsTrusted, aBuilder, aShouldDelayBuilding) {
		//_log("getDatasource");
		aShouldDelayBuilding = true;
		
		if (!aIsTrusted)
			return null;
		
		let connection = Services.storage.openSpecialDatabase("memory");
		connection.createTable("queryprocessor_sample", "url TEXT, referrer TEXT");
		return connection;
	},
	
	initializeForBuilding: function QP_initializeForBuilding(aDatasource, aBuilder, aRootNode) {
		//_log("initializeForBuilding");
		if (this._generationStarted)
			throw Cr.NS_ERROR_UNEXPECTED;
		
		this._connection = aDatasource.QueryInterface(Ci.mozIStorageConnection);
		if (!this._connection)
			throw Cr.NS_ERROR_INVALID_ARG;
		
		if (!this._connection.connectionReady)
			throw Cr.NS_ERROR_UNEXPECTED;
	},
	
	done: function QP_done() {
		//_log("done");
		this._generationStarted = false;
	},
	
	compileQuery: function QP_compileQuery(aBuilder, aQueryNode, aRefVariable, aMemberVariable) {
		//_log("compileQuery");
		let statement = this._connection.createStatement(aQueryNode.textContent);
		return statement;
	},
	
	generateResults: function QP_generateResults(aDatasource, aRef, aQuery) {
		//_log("generateResults");
		this._generationStarted = true;
		let statement = aQuery.QueryInterface(Ci.mozIStorageStatement);
		return new TemplateResultSet(statement);
	},
	
	addBinding: function QP_addBinding(aRuleNode, aVar, aVar, aExpr) {
	},
	
	translateRef: function QP_translateRef(aDatasource, aRefString) {
		//_log("translateRef");
		return new TemplateResult(null);
	},
	
	compareResults: function QP_compareResults(aLeft, aRight, aVar, aSortHints) {
		//_log("compareResults");
		let leftVal, rightVal;
		if (aLeft)
			leftVal = aLeft.getBindingFor(aVar);
		if (aRight)
			rightVal = aRight.getBindingFor(aVar);
		
		if (leftVal > rightVal)
			return 1;
		if (leftVal < rightVal)
			return -1;
		if (leftVal == rightVal)
			return 0;
	},
	
	classID: Components.ID("{247656dc-dd35-47f7-8067-38960eb31848}"),
	QueryInterface: XPCOMUtils.generateQI([Ci.nsIXULTemplateQueryProcessor])
};

this.NSGetFactory = XPCOMUtils.generateNSGetFactory([QueryProcessor]);